program Dis51;                   {Disassemble into 8051 assembly}
                                 {Source is Intel-file}

uses Dos,Crt;

{'Mnem' contains the translation for the mnemonics. The different items
 are seperated with a minus sign ('-'). An uppercase letter is 'just' an
 ASCII token. Only the lowercase letters 'mean' something. For example;
 the letter 'b' means that this place is reserved for the bit-address.}

const
     Data         = 'a';                    {Data 1 byte}
     Bit_Address  = 'b';                    {Bit address 1 byte}
     Code_Address = 'c';                    {Code address 1 byte}
     Data_Address = 'd';                    {Data address 1 byte}
     Data2        = 'e';                    {Data 2 bytes}
     Code2        = 'f';                    {Code address 2 bytes}
     Code3        = 'g';                    {Code address 11 bits}
     Data_Address2= 'h';                    {Get next code (text only)}
     Data_Address3= 'i';                    {Get previous code (text only)}
     HexString    = '0123456789ABCDEF';

var
   Mnem           : array[$00..$FF] of string[16];
   InputFile      : string[80];
   OutputFile     : string[80];
   Point          : integer;
   FromBinInput   : file of byte;
   FromInput      : text;
   ToOutput       : text;
   RealInput      : SearchRec;
   FileData       : array[$0000..$7FFF] of byte;
   TotalBytes     : word;
   StartAddress   : word;
   Temp           : string;
   SFRString      : array[0..127] of string[7];
   Loop           : byte;
   Options        : string;
   FAddress,
   FByte,
   FComment,
   FScreen,
   FBinary        : boolean;

procedure Error(ErrorNr : byte);
begin
     case ErrorNr of
       1 : writeln('Incorrect header.');
       2 : begin
             writeln('Invalid option -->     -abcsi');
             writeln('                       -a   : no address');
             writeln('                       -b   : no bytes');
             writeln('                       -c   : no comments');
             writeln('                       -s   : nothing to screen');
             writeln('                       -i   : use Intel Intellec');
           end;
       3 : writeln('Too many parameters -->  DIS8051 <inputfile> <outputfile> <options>');
       else writeln('Unknown error.');
     end;
     Halt;
end;


procedure ToChar(Value : byte;
                 var HexChar : string);
begin
  HexChar := Copy(HexString,(Value div 16)+1,1);
  HexChar := HexChar + Copy(HexString,(Value mod 16)+1,1);
end;


procedure Disas;                            {Disassemble}
var
   Pointer       : word;
   OutputString  : string[80];
   ByteString,
   MnemString,
   CharString    : string[27];
   Loop          : byte;
   Work          : byte;
   IsPos         : integer;
   SubString     : string[16];
   Index         : byte;
   TempAddress   : word;

begin
   FileData[TotalBytes+1] := 0;
   FileData[TotalBytes+2] := 0;
   Pointer := 1;
   writeln(ToOutput);
   write(ToOutput,'                        .ORG    $');
   ToChar((StartAddress div 256),Temp);
   write(ToOutput,Temp);
   ToChar((StartAddress mod 256),Temp);
   writeln(ToOutput,Temp);
   repeat
     ToChar((StartAddress div 256),Temp);
     if FScreen then
       begin
         GotoXY(20,WhereY);
         write('$',Temp);
       end;
     if FAddress then ByteString := 'L'+Temp;
     if FComment then CharString := ';'
                 else CharString := '';
     ToChar((StartAddress mod 256),Temp);
     if FScreen then write(Temp);
     if FAddress then ByteString := ByteString+Temp+'   '
                 else ByteString := '        ';
     Index := 1;
     Work := FileData[Pointer];
     Inc(Pointer);
     Inc(StartAddress);
     ToChar(Work,Temp);
     if FByte then ByteString := ByteString+Temp+' ';
     if FComment then
       if (Work<ord(' ')) or (Work>ord('z'))
         then CharString := CharString+'.'
         else CharString := CharString+chr(Work);
     MnemString := '';
     SubString := Mnem[Work];
     repeat
       IsPos := Pos('-',SubString);
       if SubString[1]>='a' then
         begin
           if SubString[1]='a' then
             begin
               Work := FileData[Pointer];
               Inc(Pointer);
               Inc(StartAddress);
               ToChar(Work,Temp);
               if FByte then ByteString := ByteString+Temp+' ';
               if FComment then
                 if (Work<ord(' ')) or (Work>ord('z'))
                   then CharString := CharString+'.'
                   else CharString := CharString+chr(Work);
               MnemString := MnemString+'$'+Temp;
             end;
           if SubString[1]='b' then
             begin
               Work := FileData[Pointer];
               Inc(Pointer);
               Inc(StartAddress);
               ToChar(Work,Temp);
               if FByte then ByteString := ByteString+Temp+' ';
               if Work<$80
                 then begin
                        ToChar(Work,Temp);
                        MnemString := MnemString+'$'+Temp;
                        Loop := (Work div 8)+$20;
                        ToChar(Loop,Temp);
                        if FComment then CharString := CharString+'($'+Temp+'.';
                        Loop := Work mod 8;
                        ToChar(Loop,Temp);
                        if FComment then CharString := CharString+Temp[2]+')';
                      end
                 else begin
                        Loop := ((Work - $80) div 8)*8;
                        MnemString := MnemString+SFRString[Loop]+'.';
                        Loop := Work mod 8;
                        ToChar(Loop,Temp);
                        MnemString := MnemString+Temp[2];
                      end;
               if FComment then
                 if (Work<ord(' ')) or (Work>ord('z'))
                   then CharString := CharString+'.'
                   else CharString := CharString+chr(Work);
             end;
           if SubString[1]='c' then
             begin
               Work := FileData[Pointer];
               Inc(Pointer);
               Inc(StartAddress);
               ToChar(Work,Temp);
               if FByte then ByteString := ByteString+Temp+' ';
               if FComment then
                 if (Work<ord(' ')) or (Work>ord('z'))
                   then CharString := CharString+'.'
                   else CharString := CharString+chr(Work);
               TempAddress := StartAddress;
               if Work>$7F then TempAddress := TempAddress-256+Work
                           else TempAddress := TempAddress+Work;
               ToChar((TempAddress div $100),Temp);
               MnemString := MnemString+'L'+Temp;
               ToChar((TempAddress mod $100),Temp);
               MnemString := MnemString+Temp;
             end;
           if SubString[1]='d' then
             begin
               Work := FileData[Pointer];
               Inc(Pointer);
               Inc(StartAddress);
               ToChar(Work,Temp);
               if FByte then ByteString := ByteString+Temp+' ';
               if FComment then
                 if (Work<ord(' ')) or (Work>ord('z'))
                   then CharString := CharString+'.'
                   else CharString := CharString+chr(Work);
               if Work<$80
                 then   MnemString := MnemString+'$'+Temp
                 else begin
                        Loop := Work - $80;
                        MnemString := MnemString+SFRString[Loop];
                      end;
             end;
           if SubString[1]='e' then
             begin
               Work := FileData[Pointer];
               Inc(Pointer);
               Inc(StartAddress);
               ToChar(Work,Temp);
               if FByte then ByteString := ByteString+Temp+' ';
               if FComment then
                 if (Work<ord(' ')) or (Work>ord('z'))
                   then CharString := CharString+'.'
                   else CharString := CharString+chr(Work);
               MnemString := MnemString+'$'+Temp;
               Work := FileData[Pointer];
               Inc(Pointer);
               Inc(StartAddress);
               ToChar(Work,Temp);
               if FByte then ByteString := ByteString+Temp+' ';
               if FComment then
                 if (Work<ord(' ')) or (Work>ord('z'))
                   then CharString := CharString+'.'
                   else CharString := CharString+chr(Work);
               MnemString := MnemString+Temp;
             end;
           if SubString[1]='f' then
             begin
               Work := FileData[Pointer];
               Inc(Pointer);
               Inc(StartAddress);
               ToChar(Work,Temp);
               if FByte then ByteString := ByteString+Temp+' ';
               if FComment then
                 if (Work<ord(' ')) or (Work>ord('z'))
                   then CharString := CharString+'.'
                   else CharString := CharString+chr(Work);
               MnemString := MnemString+'L'+Temp;
               Work := FileData[Pointer];
               Inc(Pointer);
               Inc(StartAddress);
               ToChar(Work,Temp);
               if FByte then ByteString := ByteString+Temp+' ';
               if FComment then
                 if (Work<ord(' ')) or (Work>ord('z'))
                   then CharString := CharString+'.'
                   else CharString := CharString+chr(Work);
               MnemString := MnemString+Temp;
             end;
           if SubString[1]='g' then
             begin
               TempAddress := Work shr 5;
               TempAddress := TempAddress shl 8;
               TempAddress := TempAddress or ((StartAddress+1) and $F800);
               Work := FileData[Pointer];
               Inc(Pointer);
               Inc(StartAddress);
               TempAddress := TempAddress + Work;
               ToChar(Work,Temp);
               if FByte then ByteString := ByteString+Temp+' ';
               if FComment then
                 if (Work<ord(' ')) or (Work>ord('z'))
                   then CharString := CharString+'.'
                   else CharString := CharString+chr(Work);
               ToChar((TempAddress div 256),Temp);
               MnemString := MnemString+'L'+Temp;
               ToChar((TempAddress mod 256),Temp);
               MnemString := MnemString+Temp;
             end;
           if SubString[1]='h' then
             begin
               Work := FileData[Pointer];
               Inc(Pointer);
               Inc(StartAddress);
               ToChar(Work,Temp);
               if FByte then ByteString := ByteString+Temp+' ';
               if FComment then
                 if (Work<ord(' ')) or (Work>ord('z'))
                   then CharString := CharString+'.'
                   else CharString := CharString+chr(Work);
               Work := FileData[Pointer];
               ToChar(Work,Temp);
               if Work<$80
                 then   MnemString := MnemString+'$'+Temp
                 else begin
                        Loop := Work - $80;
                        MnemString := MnemString+SFRString[Loop];
                      end;
             end;
           if SubString[1]='i' then
             begin
               Work := FileData[Pointer];
               Inc(Pointer);
               Inc(StartAddress);
               ToChar(Work,Temp);
               if FByte then ByteString := ByteString+Temp+' ';
               if FComment then
                 if (Work<ord(' ')) or (Work>ord('z'))
                   then CharString := CharString+'.'
                   else CharString := CharString+chr(Work);
               Work := FileData[Pointer-2];
               ToChar(Work,Temp);
               if Work<$80
                 then   MnemString := MnemString+'$'+Temp
                 else begin
                        Loop := Work - $80;
                        MnemString := MnemString+SFRString[Loop];
                      end;
             end;
         end
         else if IsPos=0 then MnemString := MnemString + SubString
                         else MnemString := MnemString + Copy(SubString,1,IsPos-1);
       if Index=1 then for Loop := Length(MnemString) to 7 do MnemString := MnemString+' ';
       Inc(Index);
       SubString := Copy(SubString,1+IsPos,Length(SubString)-IsPos);
     until IsPos=0;
     OutputString := ByteString;
     for Loop := Length(ByteString) to 23 do OutputString := OutputString+' ';
     for Loop := Length(MnemString) to 23 do MnemString := MnemString+' ';
     OutputString := OutputString+MnemString+CharString;
     writeln(ToOutput,OutputString);
   until Pointer>TotalBytes;
end;

function ToNum(First, Second : char): byte; {Convert characters to num}
begin
     ToNum := ((Pos(First,HexString)-1) * 16) + Pos(Second,HexString)-1;
end;


procedure GoReadData;                       {Read input file into memory}
var
   NrBytes : byte;
   OneLine : string;
   TheAddress,
   TargetAddress : word;
   Loop          : byte;
   Loop2         : byte;
   Loop3         : byte;
   Temp2         : string;
   OneByte       : byte;

begin
   if FBinary then
     begin
       assign(FromBinInput, InputFile);
       reset(FromBinInput);
     end
     else
     begin
       assign(FromInput,InputFile);
       reset(FromInput);
     end;
   assign(ToOutput,OutputFile);
   rewrite(ToOutput);
   writeln(ToOutput,'; Disassembler  Intel MCS-51');
   writeln(ToOutput,';(C)opyright 1989-1996 by M.Majoor');
   writeln(ToOutput);
   writeln(ToOutput,';Special function registers');
   for Loop := 0 to $7F do
     if SFRString[Loop]<>'ILLEGAL' then
       begin
         Temp2 := SFRString[Loop];
         for Loop2 := Length(Temp2) to 23 do Temp2 := Temp2+' ';
         Temp2 := Temp2+'.EQU    $';
         ToChar((Loop+$80),Temp);
         Temp2 := Temp2+Temp;
         writeln(ToOutput,Temp2);
       end;
   writeln(ToOutput);
   writeln(ToOutput,';Bit addressable registers');
   Loop := 0;
   repeat
     if SFRString[Loop]<>'ILLEGAL' then
       for Loop2 := 0 to 7 do
         begin
           Temp2 := SFRString[Loop]+'.';
           ToChar(Loop2,Temp);
           Temp2 := Temp2+Temp[2];
           for Loop3 := Length(Temp2) to 23 do Temp2 := Temp2+' ';
           Temp2 := Temp2+'.EQU    $';
           ToChar((Loop+Loop2+$80),Temp);
           Temp2 := Temp2+Temp;
           writeln(ToOutput,Temp2);
         end;
     Loop := Loop + $08;
   until Loop>$7F;
   writeln(ToOutput);
   if FScreen then writeln('Reading address    Disassemble address');

   if FBinary then
     begin
       TotalBytes := 0;
       TargetAddress := 0;
       while (not(Eof(FromBinInput))) do
         begin
           read(FromBinInput, OneByte);
           if FScreen then
             begin
               GotoXY(1,WhereY);
               ToChar(((TargetAddress+NrBytes-1) div 256),Temp);
               write('$',Temp);
               ToChar(((TargetAddress+NrBytes-1) mod 256),Temp);
               write(Temp);
             end;
           Inc(TotalBytes);
           FileData[TotalBytes] := OneByte;
           Inc(TargetAddress);
         end;
       StartAddress := 0;
       Disas;
     end
     else
     begin
       TotalBytes := 0;
       TargetAddress := $FFFF;
       OneLine[8] := '0';
       OneLine[9] := '0';
       while (not(Eof(FromInput)) and (ToNum(OneLine[8],OneLine[9])=0)) do
         begin
           readln(FromInput,OneLine);
           if OneLine[1]<>':' then Error(1);
           NrBytes := ToNum(OneLine[2],OneLine[3]);
           TheAddress := ToNum(OneLine[4],OneLine[5]);
           TheAddress := TheAddress * $100 + ToNum(OneLine[6],OneLine[7]);
           if TargetAddress=$FFFF then
             begin
               TargetAddress := TheAddress;
               StartAddress  := TheAddress;
             end;
           if TargetAddress=TheAddress then
             if FScreen then
               begin
                 GotoXY(1,WhereY);
                 ToChar(((TargetAddress+NrBytes-1) div 256),Temp);
                 write('$',Temp);
                 ToChar(((TargetAddress+NrBytes-1) mod 256),Temp);
                 write(Temp);
               end;
           if ToNum(OneLine[8],OneLine[9])=1
             then Disas
             else begin
                    if TargetAddress<>TheAddress
                      then
                      begin
                        Disas;
                        StartAddress := TheAddress;
                        TotalBytes    := 0;
                      end;
                    for Loop := 1 to NrBytes do
                      begin
                        Inc(TotalBytes);
                        FileData[TotalBytes] := ToNum(OneLine[Loop*2+8],OneLine[Loop*2+9]);
                      end;
                  end;
             if TargetAddress<>$FFFF then TargetAddress := TheAddress + NrBytes;
         end;
     end;
   writeln(ToOutput);
   writeln(ToOutput,'                        .END');
   close(ToOutput);
   if FBinary then close(FromBinInput)
              else close(FromInput);
end;


begin
     writeln;
     writeln('Disassembler for Intel MCS-51 series.');
     writeln('Binary file  or  Intel Intellec file');
     writeln('(C)opyright 1989-1996 by M.Majoor');
     writeln;
     FAddress := true;
     FByte    := true;
     FComment := true;
     FScreen  := true;
     FBinary  := true;
     case ParamCount of
          0 : begin
                write('Input  file : ');
                readln(InputFile);
                write('Output file : ');
                readln(OutputFile);
                write('Options     : ');
                readln(Options);
              end;
          1 : begin
                InputFile := ParamStr(1);
                write('Output file : ');
                readln(OutputFile);
                write('Options     : ');
                readln(Options);
              end;
          2 : begin
                InputFile  := ParamStr(1);
                OutputFile := ParamStr(2);
                write('Options : ');
                readln(Options);
              end;
          3 : begin
                InputFile  := ParamStr(1);
                OutputFile := ParamStr(2);
                Options    := ParamStr(3);
              end;
         else Error(3);
     end;
     if InputFile = '' then
       begin
         writeln('No input file specified.');
         Exit;
       end;
     FindFirst(InputFile,0,RealInput);
     if Doserror<>0 then
       begin
         case Doserror of
           2 : writeln('File not found');
           3 : writeln('Path not found');
           5 : writeln('Access denied');
           6 : writeln('Invalid handle');
           8 : writeln('Not enough memory');
          10 : writeln('Invalid environment');
          11 : writeln('Invalid format');
          18 : writeln('No more files');
         end;
         Exit;
       end;
     InputFile := RealInput.Name;
     if OutputFile = '' then
       begin
         Point := Pos('.',InputFile);
         if Point=0 then OutputFile := InputFile + '.51'
                    else OutputFile := Copy(InputFile,1,Point-1)+'.51';
       end;
     if Options<>'' then
       begin
         if Options[1]<>'-' then Error(2);
         if Length(Options)<2 then Error(2);
         for Loop := 2 to Length(Options) do
           case Options[Loop] of
              'a','A' : FAddress := false;
              'b','B' : FByte    := false;
              'c','C' : FComment := false;
              's','S' : FScreen  := false;
              'i','I' : FBinary   := false
              else Error(2);
           end;
       end;
     writeln;
     writeln('Disassembly of <',InputFile,'> to <',OutputFile,'>');
     Mnem[$00] := 'NOP';
     Mnem[$01] := 'AJMP-g';
     Mnem[$02] := 'LJMP-f';
     Mnem[$03] := 'RR-A';
     Mnem[$04] := 'INC-A';
     Mnem[$05] := 'INC-d';
     Mnem[$06] := 'INC-@R0';
     Mnem[$07] := 'INC-@R1';
     Mnem[$08] := 'INC-R0';
     Mnem[$09] := 'INC-R1';
     Mnem[$0A] := 'INC-R2';
     Mnem[$0B] := 'INC-R3';
     Mnem[$0C] := 'INC-R4';
     Mnem[$0D] := 'INC-R5';
     Mnem[$0E] := 'INC-R6';
     Mnem[$0F] := 'INC-R7';

     Mnem[$10] := 'JBC-b-,-c';
     Mnem[$11] := 'ACALL-g';
     Mnem[$12] := 'LCALL-f';
     Mnem[$13] := 'RRC-A';
     Mnem[$14] := 'DEC-A';
     Mnem[$15] := 'DEC-d';
     Mnem[$16] := 'DEC-@R0';
     Mnem[$17] := 'DEC-@R1';
     Mnem[$18] := 'DEC-R0';
     Mnem[$19] := 'DEC-R1';
     Mnem[$1A] := 'DEC-R2';
     Mnem[$1B] := 'DEC-R3';
     Mnem[$1C] := 'DEC-R4';
     Mnem[$1D] := 'DEC-R5';
     Mnem[$1E] := 'DEC-R6';
     Mnem[$1F] := 'DEC-R7';

     Mnem[$20] := 'JB-b-,-c';
     Mnem[$21] := 'AJMP-g';
     Mnem[$22] := 'RET';
     Mnem[$23] := 'RL-A';
     Mnem[$24] := 'ADD-A,#-a';
     Mnem[$25] := 'ADD-A,-d';
     Mnem[$26] := 'ADD-A,@R0';
     Mnem[$27] := 'ADD-A,@R1';
     Mnem[$28] := 'ADD-A,R0';
     Mnem[$29] := 'ADD-A,R1';
     Mnem[$2A] := 'ADD-A,R2';
     Mnem[$2B] := 'ADD-A,R3';
     Mnem[$2C] := 'ADD-A,R4';
     Mnem[$2D] := 'ADD-A,R5';
     Mnem[$2E] := 'ADD-A,R6';
     Mnem[$2F] := 'ADD-A,R7';

     Mnem[$30] := 'JNB-b-,-c';
     Mnem[$31] := 'ACALL-g';
     Mnem[$32] := 'RETI';
     Mnem[$33] := 'RLC-A';
     Mnem[$34] := 'ADDC-A,#-a';
     Mnem[$35] := 'ADDC-A,-d';
     Mnem[$36] := 'ADDC-A,@R0';
     Mnem[$37] := 'ADDC-A,@R1';
     Mnem[$38] := 'ADDC-A,R0';
     Mnem[$39] := 'ADDC-A,R1';
     Mnem[$3A] := 'ADDC-A,R2';
     Mnem[$3B] := 'ADDC-A,R3';
     Mnem[$3C] := 'ADDC-A,R4';
     Mnem[$3D] := 'ADDC-A,R5';
     Mnem[$3E] := 'ADDC-A,R6';
     Mnem[$3F] := 'ADDC-A,R7';

     Mnem[$40] := 'JC-c';
     Mnem[$41] := 'AJMP-g';
     Mnem[$42] := 'ORL-d-,A';
     Mnem[$43] := 'ORL-d-,#-a';
     Mnem[$44] := 'ORL-A,#-a';
     Mnem[$45] := 'ORL-A,-d';
     Mnem[$46] := 'ORL-A,@R0';
     Mnem[$47] := 'ORL-A,@R1';
     Mnem[$48] := 'ORL-A,R0';
     Mnem[$49] := 'ORL-A,R1';
     Mnem[$4A] := 'ORL-A,R2';
     Mnem[$4B] := 'ORL-A,R3';
     Mnem[$4C] := 'ORL-A,R4';
     Mnem[$4D] := 'ORL-A,R5';
     Mnem[$4E] := 'ORL-A,R6';
     Mnem[$4F] := 'ORL-A,R7';

     Mnem[$50] := 'JNC-c';
     Mnem[$51] := 'ACALL-g';
     Mnem[$52] := 'ANL-d-,A';
     Mnem[$53] := 'ANL-d-,#-a';
     Mnem[$54] := 'ANL-A,#-a';
     Mnem[$55] := 'ANL-A,-d';
     Mnem[$56] := 'ANL-A,@R0';
     Mnem[$57] := 'ANL-A,@R1';
     Mnem[$58] := 'ANL-A,R0';
     Mnem[$59] := 'ANL-A,R1';
     Mnem[$5A] := 'ANL-A,R2';
     Mnem[$5B] := 'ANL-A,R3';
     Mnem[$5C] := 'ANL-A,R4';
     Mnem[$5D] := 'ANL-A,R5';
     Mnem[$5E] := 'ANL-A,R6';
     Mnem[$5F] := 'ANL-A,R7';

     Mnem[$60] := 'JZ-c';
     Mnem[$61] := 'AJMP-g';
     Mnem[$62] := 'XRL-d-,A';
     Mnem[$63] := 'XRL-d-,#-a';
     Mnem[$64] := 'XRL-A,#-a';
     Mnem[$65] := 'XRL-A,-d';
     Mnem[$66] := 'XRL-A,@R0';
     Mnem[$67] := 'XRL-A,@R1';
     Mnem[$68] := 'XRL-A,R0';
     Mnem[$69] := 'XRL-A,R1';
     Mnem[$6A] := 'XRL-A,R2';
     Mnem[$6B] := 'XRL-A,R3';
     Mnem[$6C] := 'XRL-A,R4';
     Mnem[$6D] := 'XRL-A,R5';
     Mnem[$6E] := 'XRL-A,R6';
     Mnem[$6F] := 'XRL-A,R7';

     Mnem[$70] := 'JNZ-c';
     Mnem[$71] := 'ACALL-g';
     Mnem[$72] := 'ORL-C,-b';
     Mnem[$73] := 'JMP-@A+DPTR';
     Mnem[$74] := 'MOV-A,#-a';
     Mnem[$75] := 'MOV-d-,#-a';
     Mnem[$76] := 'MOV-@R0,#-a';
     Mnem[$77] := 'MOV-@R1,#-a';
     Mnem[$78] := 'MOV-R0,#-a';
     Mnem[$79] := 'MOV-R1,#-a';
     Mnem[$7A] := 'MOV-R2,#-a';
     Mnem[$7B] := 'MOV-R3,#-a';
     Mnem[$7C] := 'MOV-R4,#-a';
     Mnem[$7D] := 'MOV-R5,#-a';
     Mnem[$7E] := 'MOV-R6,#-a';
     Mnem[$7F] := 'MOV-R7,#-a';

     Mnem[$80] := 'SJMP-c';
     Mnem[$81] := 'AJMP-g';
     Mnem[$82] := 'ANL-C,-b';
     Mnem[$83] := 'MOVC-A,@A+PC';
     Mnem[$84] := 'DIV-AB';
     Mnem[$85] := 'MOV-h-,-i';
     Mnem[$86] := 'MOV-d-,@R0';
     Mnem[$87] := 'MOV-d-,@R1';
     Mnem[$88] := 'MOV-d-,R0';
     Mnem[$89] := 'MOV-d-,R1';
     Mnem[$8A] := 'MOV-d-,R2';
     Mnem[$8B] := 'MOV-d-,R3';
     Mnem[$8C] := 'MOV-d-,R4';
     Mnem[$8D] := 'MOV-d-,R5';
     Mnem[$8E] := 'MOV-d-,R6';
     Mnem[$8F] := 'MOV-d-,R7';

     Mnem[$90] := 'MOV-DPTR,#-e';
     Mnem[$91] := 'ACALL-g';
     Mnem[$92] := 'MOV-b-,C';
     Mnem[$93] := 'MOVC-A,@A+DPTR';
     Mnem[$94] := 'SUBB-A,#-a';
     Mnem[$95] := 'SUBB-A,-d';
     Mnem[$96] := 'SUBB-A,@R0';
     Mnem[$97] := 'SUBB-A,@R1';
     Mnem[$98] := 'SUBB-A,R0';
     Mnem[$99] := 'SUBB-A,R1';
     Mnem[$9A] := 'SUBB-A,R2';
     Mnem[$9B] := 'SUBB-A,R3';
     Mnem[$9C] := 'SUBB-A,R4';
     Mnem[$9D] := 'SUBB-A,R5';
     Mnem[$9E] := 'SUBB-A,R6';
     Mnem[$9F] := 'SUBB-A,R7';

     Mnem[$A0] := 'ORL-C,/-b';
     Mnem[$A1] := 'AJMP-g';
     Mnem[$A2] := 'MOV-C,-b';
     Mnem[$A3] := 'INC-DPTR';
     Mnem[$A4] := 'MUL-AB';
     Mnem[$A5] := 'RESERVED';
     Mnem[$A6] := 'MOV-@R0,-d';
     Mnem[$A7] := 'MOV-@R1,-d';
     Mnem[$A8] := 'MOV-R0,-d';
     Mnem[$A9] := 'MOV-R1,-d';
     Mnem[$AA] := 'MOV-R2,-d';
     Mnem[$AB] := 'MOV-R3,-d';
     Mnem[$AC] := 'MOV-R4,-d';
     Mnem[$AD] := 'MOV-R5,-d';
     Mnem[$AE] := 'MOV-R6,-d';
     Mnem[$AF] := 'MOV-R7,-d';

     Mnem[$B0] := 'ANL-C,/-b';
     Mnem[$B1] := 'ACALL-g';
     Mnem[$B2] := 'CPL-b';
     Mnem[$B3] := 'CPL-C';
     Mnem[$B4] := 'CJNE-A,#-a-,-c';
     Mnem[$B5] := 'CJNE-A,-d-,-c';
     Mnem[$B6] := 'CJNE-@R0,#-a-,-c';
     Mnem[$B7] := 'CJNE-@R1,#-a-,-c';
     Mnem[$B8] := 'CJNE-R0,#-a-,-c';
     Mnem[$B9] := 'CJNE-R1,#-a-,-c';
     Mnem[$BA] := 'CJNE-R2,#-a-,-c';
     Mnem[$BB] := 'CJNE-R3,#-a-,-c';
     Mnem[$BC] := 'CJNE-R4,#-a-,-c';
     Mnem[$BD] := 'CJNE-R5,#-a-,-c';
     Mnem[$BE] := 'CJNE-R6,#-a-,-c';
     Mnem[$BF] := 'CJNE-R7,#-a-,-c';

     Mnem[$C0] := 'PUSH-d';
     Mnem[$C1] := 'AJMP-g';
     Mnem[$C2] := 'CLR-b';
     Mnem[$C3] := 'CLR-C';
     Mnem[$C4] := 'SWAP-A';
     Mnem[$C5] := 'XCH-A,-d';
     Mnem[$C6] := 'XCH-A,@R0';
     Mnem[$C7] := 'XCH-A,@R1';
     Mnem[$C8] := 'XCH-A,R0';
     Mnem[$C9] := 'XCH-A,R1';
     Mnem[$CA] := 'XCH-A,R2';
     Mnem[$CB] := 'XCH-A,R3';
     Mnem[$CC] := 'XCH-A,R4';
     Mnem[$CD] := 'XCH-A,R5';
     Mnem[$CE] := 'XCH-A,R6';
     Mnem[$CF] := 'XCH-A,R7';

     Mnem[$D0] := 'POP-d';
     Mnem[$D1] := 'ACALL-g';
     Mnem[$D2] := 'SETB-b';
     Mnem[$D3] := 'SETB-C';
     Mnem[$D4] := 'DA-A';
     Mnem[$D5] := 'DJNZ-d-,-c';
     Mnem[$D6] := 'XCHD-A,@R0';
     Mnem[$D7] := 'XCHD-A,@R1';
     Mnem[$D8] := 'DJNZ-R0,-c';
     Mnem[$D9] := 'DJNZ-R1,-c';
     Mnem[$DA] := 'DJNZ-R2,-c';
     Mnem[$DB] := 'DJNZ-R3,-c';
     Mnem[$DC] := 'DJNZ-R4,-c';
     Mnem[$DD] := 'DJNZ-R5,-c';
     Mnem[$DE] := 'DJNZ-R6,-c';
     Mnem[$DF] := 'DJNZ-R7,-c';

     Mnem[$E0] := 'MOVX-A,@DPTR';
     Mnem[$E1] := 'AJMP-g';
     Mnem[$E2] := 'MOVX-A,@R0';
     Mnem[$E3] := 'MOVX-A,@R1';
     Mnem[$E4] := 'CLR-A';
     Mnem[$E5] := 'MOV-A,-d';
     Mnem[$E6] := 'MOV-A,@R0';
     Mnem[$E7] := 'MOV-A,@R1';
     Mnem[$E8] := 'MOV-A,R0';
     Mnem[$E9] := 'MOV-A,R1';
     Mnem[$EA] := 'MOV-A,R2';
     Mnem[$EB] := 'MOV-A,R3';
     Mnem[$EC] := 'MOV-A,R4';
     Mnem[$ED] := 'MOV-A,R5';
     Mnem[$EE] := 'MOV-A,R6';
     Mnem[$EF] := 'MOV-A,R7';

     Mnem[$F0] := 'MOVX-@DPTR,A';
     Mnem[$F1] := 'ACALL-g';
     Mnem[$F2] := 'MOVX-@R0,A';
     Mnem[$F3] := 'MOVX-@R1,A';
     Mnem[$F4] := 'CPL-A';
     Mnem[$F5] := 'MOV-d-,A';
     Mnem[$F6] := 'MOV-@R0,A';
     Mnem[$F7] := 'MOV-@R1,A';
     Mnem[$F8] := 'MOV-R0,A';
     Mnem[$F9] := 'MOV-R1,A';
     Mnem[$FA] := 'MOV-R2,A';
     Mnem[$FB] := 'MOV-R3,A';
     Mnem[$FC] := 'MOV-R4,A';
     Mnem[$FD] := 'MOV-R5,A';
     Mnem[$FE] := 'MOV-R6,A';
     Mnem[$FF] := 'MOV-R7,A';

     for Loop := 0 to $7F do SFRString[Loop] := 'ILLEGAL';
     SFRString[$00] := 'P0';
     SFRString[$01] := 'SP';
     SFRString[$02] := 'DPL';
     SFRString[$03] := 'DPH';
     SFRString[$07] := 'PCON';
     SFRString[$08] := 'TCON';
     SFRString[$09] := 'TMOD';
     SFRString[$0A] := 'TL0';
     SFRString[$0B] := 'TL1';
     SFRString[$0C] := 'TH0';
     SFRString[$0D] := 'TH1';
     SFRString[$10] := 'P1';
     SFRString[$18] := 'SCON';
     SFRString[$19] := 'SBUF';
     SFRString[$20] := 'P2';
     SFRString[$28] := 'IE';
     SFRString[$30] := 'P3';
     SFRString[$38] := 'IP';
     SFRString[$48] := 'T2CON';
     SFRString[$4A] := 'RCAP2L';
     SFRString[$4B] := 'RCAP2H';
     SFRString[$4C] := 'TL2';
     SFRString[$4D] := 'TH2';
     SFRString[$50] := 'PSW';
     SFRString[$58] := 'ADCON';
     SFRString[$60] := 'ACC';
     SFRString[$68] := 'P4';
     SFRString[$70] := 'B';
     SFRString[$78] := 'P5';
     GoReadData;
     writeln;
end.