program DiskIm2;

{ Creates a disk image of **any** disk. Optionally a start sector and
  number of sectors to read can be given.
  Note: the DOS call to retrieve the number of sectors is actually not
        correct, because it only returns the number of available
        sectors for DOS. The FAT and bootsector are not included in this
        count!!
}
uses Crt;

const
  TransferMax = $8000;
type
  TheBuffer = array[0..TransferMax-1] of byte;
  aList = record
            SectorNumber   : longint;
            SectorsToRead  : word;
            TransferAddress: pointer;
          end;

var
  strFromDisk,
  strToFile,
  strStartingSector,
  strNumberOfSectors: string;
  TransferBuffer    : TheBuffer;
  TransferSize      : word;
  ToWrite           : word;
  Written           : word;
  TransferSectors   : word;

  CurrentSector     : longint;
  StartSector       : longint;
  LastSector        : longint;
  ErrorStr          : integer;
  ToFile            : file;
  Error             : word;
  ErrorResult       : word;
  Percentage        : longint;
  Hundred           : longint;

  FromDisk          : byte;
  NumberOfSectors   : word;


  Clusters          : word;
  SectorCluster     : byte;
  BytesSector       : word;
  MaxSector         : longint;
  Size              : longint;
  strSize           : string;
  strSize2          : string;
  LengthStr         : byte;
  IndexStr          : byte;
  ParameterList     : aList;
  TheKey            : char;

{ Read the indicated number of sectors. Returns error code.
  The parameters must be set correctly! }
function ReadAbsolute(Drive: byte; Parameters : pointer): word;
begin
  asm
    push ds
    mov  ax, [BP+offset Parameters+2]
    push ax
    pop  ds
    mov  al, [BP+offset Drive]
    mov  bx, [BP+offset Parameters]
    mov  cx, $FFFF                 { Use >32M format }
    int  $25
    pop  bx                        { Remove saved flags }
    jc   @NoError
    mov  @Result, ax
    jmp  @Exit
  @NoError:
    mov  @Result, 0
  @Exit:
    pop  ds
  end;
end;


procedure GetFATInformation(Drive: byte);
begin
  asm
    push ds
    mov  dl, Drive
    inc  dl                     { +1 so not 'current' drive }
    mov  ah, $1C
    int  $21
    pop  ds
    mov  Clusters, dx
    mov  SectorCluster, al
    mov  BytesSector, cx
  end;
end;


begin
  writeln('Disk To Image          (C)opyright 1997 by M.Majoor          Disk To Image');
  writeln;
  writeln('                  Creates an image-file of a given disk.');
  writeln;
  write('Disk to create image for         : ');
  readln(strFromDisk);
  if strFromDisk[0]<>#1 then
    begin
      writeln('Incorrect name entry, use one character only.');
      Halt;
    end;
  if not(strFromDisk[1] in ['A'..'Z', 'a'..'z']) then
    begin
      writeln('Incorrect name entry, use identifiers like A, B, C etc only.');
      Halt;
    end;
  FromDisk := ord(upcase(strFromDisk[1])) - ord('A');

  GetFATInformation(FromDisk);
  if SectorCluster=$FF then
    begin
      writeln('Drive ', chr(FromDisk + ord('A')), ' does not exist or is not present.');
      Halt;
    end;
  MaxSector := longint(Clusters) * SectorCluster - 1;
  writeln;
  writeln('Drive ', chr(FromDisk + ord('A')), ' parameters:');
  writeln('  Clusters        : ', Clusters);
  writeln('  Sectors         : ', MaxSector+1);
  writeln('  Sectors/cluster : ', SectorCluster);
  writeln('  Bytes/sector    : ', BytesSector);
  Size := longint(BytesSector) * SectorCluster * Clusters;

  str(Size, strSize);
  LengthStr := ord(strSize[0]);
  IndexStr  := LengthStr;
  strSize2  := '';
  while IndexStr<>0 do
    begin
      strSize2 := strSize[IndexStr] + strSize2;
      dec(IndexStr);
      if ((LengthStr - IndexStr) mod 3)=0 then strSize2 := '.' + strSize2;
    end;
  while strSize2[1]='.' do delete(strSize2, 1, 1);
  writeln('  Size            : ', strSize2, ' bytes');
{  writeln('  Starting sector : ', CurrentSector);}
{  writeln('  Ending sector   : ', LastSector);   }

  writeln;
  if Clusters=$FFFF then
    begin
      writeln('Drive ', chr(FromDisk + ord('A')), ' number of clusters incorrect (CD-ROM?).');
      Halt;
    end;

  write('File to save image to            : ');
  readln(strToFile);
  if strToFile='' then
    begin
      writeln('No destination filename specified.');
      halt;
    end;
  write('Starting sector (0=begin)        : ');
  readln(strStartingSector);
  if strStartingSector='' then strStartingSector := '0';
  val(strStartingSector, CurrentSector, ErrorStr);
  if ErrorStr<>0 then
    begin
      writeln('Invalid starting sector.');
      halt;
    end;
  if ((CurrentSector<0) or (CurrentSector>MaxSector)) then
    begin
      writeln('Starting sector too small or too large.');
      halt;
    end;
  StartSector := CurrentSector;
  write('Number of sectors to read (0=all): ');
  readln(strNumberOfSectors);
  if strNumberOfSectors='' then strNumberOfSectors := '0';
  val(strNumberOfSectors, LastSector, ErrorStr);
  if ErrorStr<>0 then
    begin
      writeln('Invalid number of sectors to read sector.');
      halt;
    end;

  if LastSector=0 then LastSector := MaxSector
                  else LastSector := CurrentSector + LastSector;
  if ((LastSector<CurrentSector) or (LastSector>MaxSector)) then
    begin
      writeln('Ending sector too small or too large.');
      halt;
    end;
  if LastSector=CurrentSector then LastSector := MaxSector
                              else LastSector := LastSector - 1;


  TransferSize := TransferMax;                   { Try default buffersize }
                                                 { Make multiple of clustersize }
  TransferSize := (TransferSize div (SectorCluster*BytesSector)) *
                                    (SectorCluster*BytesSector);
  TransferSectors := TransferSize div BytesSector;
{  writeln('  Transferbuffer : ', TransferSize, ' bytes');      }
{  writeln('  Sectors buffer : ', TransferSectors, ' sectors'); }

  {$I-}
  assign(ToFile, strToFile);
  rewrite(ToFile, 1);
  {$I+}
  ErrorResult := IoResult;
  if ErrorResult<>0 then
    begin
      writeln('Error creating output file ''', strToFile, '''.');
      halt;
    end;

  write('Writing to destination ...    %'#8#8#8#8);
  ParameterList.TransferAddress := @TransferBuffer;
  if LastSector>MaxSector then LastSector := MaxSector;
  Hundred := LastSector - StartSector;
  repeat
    NumberOfSectors := TransferSectors;
    if (CurrentSector+NumberOfSectors)>LastSector
      then NumberOfSectors := LastSector-CurrentSector+1;
    with ParameterList do
      begin
          SectorNumber    := CurrentSector;
          SectorsToRead   := NumberOfSectors;
        end;
    Error := ReadAbsolute(FromDisk, @ParameterList);
    if Error=0 then
      begin
        ToWrite := NumberOfSectors*BytesSector;
        BlockWrite(ToFile, TransferBuffer, ToWrite, Written);
        if ToWrite<>Written then Error := $FFFF;
      end;
    CurrentSector := CurrentSector + NumberOfSectors;
    if Hundred>0 then Percentage := ((CurrentSector-1) - StartSector)*100 div Hundred
                 else Percentage := 0;
    write(Percentage:3, #8#8#8);
    if KeyPressed then
      begin
        TheKey := ReadKey;
        if TheKey=#3 then Error := $FFFE;
      end;
  until (CurrentSector>LastSector) or (Error<>0);
  close(ToFile);
  if Error<>0 then
    case Error of
      $FFFF: begin writeln; writeln('Error occured during writing destination.'); end;
      $FFFE: begin writeln; writeln('Interrupted'); end;
      else writeln('Error ', Error);
    end;
end.