program Emul;                     {EPROM emulator software}
uses crt,dos;


const
  PortAddr = $378;

  DataReg  = $00;
  StatReg  = $01;
  CtrlReg  = $02;

  Dir      = $20;                 {Direction databus (0=output)}
  Select   = $08;                 {SelectIn (inverted on the output)}
  Init     = $04;                 {Init}
  Latch    = $02;                 {AutoFeed (inverted on the output)}
  Strobe   = $01;                 {Strobe   (inverted on the output)}

var
  Control : byte;                 {Internally used status of CtrlReg}

{Give strobe signal (for reading purposes also returns data)}
procedure WriteStrobe;
begin
  Port[PortAddr+CtrlReg] := Control or Strobe;   {Strobe low }
  Port[PortAddr+CtrlReg] := Control;             {Strobe high}
end;


{Give strobe signal and read data}
function ReadStrobe: byte;
begin
  Port[PortAddr+CtrlReg] := Control or Strobe;   {Strobe low }
  ReadStrobe := Port[PortAddr+DataReg];
  Port[PortAddr+CtrlReg] := Control;             {Strobe high}
end;


{Enable target system}
procedure EnableTarget;
begin
  Control := Control or Select;
  Port[PortAddr+CtrlReg] := Control;             {Select low}
end;                                     


{Enable local system}
procedure EnableLocal;
begin
  Control := Control and not(Select);
  Port[PortAddr+CtrlReg] := Control;             {Select high}
end;


{Write address to latches}
procedure WriteAddress(TheAddress: word);
begin
  Port[PortAddr+DataReg] := lo(TheAddress);      {Set up data}
  Control := Control or Latch;                   {AutoFd low}
  Control := Control and not(Init);              {Init low}
  Port[PortAddr+CtrlReg] := Control;
  WriteStrobe;                                   {Latch to low address latch}
  Port[PortAddr+DataReg] := hi(TheAddress);      {Set up data}
  Control := Control or Init;                    {Init high}
  Port[PortAddr+CtrlReg] := Control;
  WriteStrobe;                                   {Latch to high address latch}
end;


{Write data to RAM}                                                                        
procedure WriteData(TheData: byte);
begin
  Port[PortAddr+DataReg] := TheData;             {Set up data}
  Control := Control and not(Latch);             {AutoFd high}
  Control := Control or Init;                    {Init high}
  Port[PortAddr+CtrlReg] := Control;
  WriteStrobe;                                   {Give write strobe}
end;


{Read data from RAM}
function ReadData: byte;
begin
  Port[PortAddr+DataReg] := $00;                 {??? ORed with input according to some doc's}
  Control := Control and not(Latch);             {AutoFd high}
  Control := Control and not(Init);              {Init low}
  Control := Control or Dir;                     {Direction}
  Port[PortAddr+CtrlReg] := Control;
  ReadData := ReadStrobe;                        {Give read strobe}
  Control := Control or Init;                    {Init high}
  Control := Control and not(Dir);               {Direction}
  Port[PortAddr+CtrlReg] := Control;             {Return direction}
end;


{Write address/data to latches/RAM}
procedure WriteDataAtAddress(TheAddress: word; TheData: byte);
begin
  WriteAddress(TheAddress);
  WriteData(TheData);
end;


{Write address/data to latches/RAM with readback}
function WriteDataReadbackAtAddress(TheAddress: word; TheData: byte): byte;
begin
  WriteAddress(TheAddress);
  WriteData(TheData);
  WriteDataReadbackAtAddress := ReadData;
end;


{Read data from address}
function ReadDataFromAddress(TheAddress: word): byte;
begin
  WriteAddress(TheAddress);
  ReadDataFromAddress := ReadData;
end;

label endit;

var
  Loop      : integer;
  Result    : byte;
  Result2   : byte;
  Large     : array[0..$7FFF] of byte;
  test      : string;

begin
  writeln;
  writeln;
  Control := Init or Select;                     {Default status control register}
  Port[PortAddr+CtrlReg] := Control;             {Set 'default' status}

  EnableLocal;
  randomize;
  for Loop := 0 to $7FFF do Large[Loop] := random($100);
repeat
  for Loop := 0 to $7FFF do
    begin
      WriteAddress(Loop);
      WriteData(Large[Loop]);
      WriteAddress(Loop);
      Result := ReadData;
      if Result<>Large[Loop] then
        begin
          write(Loop,'=',Large[Loop],'=',Result,' ');
        end;
if keypressed then goto endit;

    end;
until keypressed;
goto endit;
  for Loop := 0 to $7FFF do
    begin
      if keypressed then halt;
      Result  := ReadDataFromAddress(Loop);
      if Result<>Large[Loop] then
        begin
          write(Loop,'=',large[loop],'=',result,' ');
        end;
    end;

endit:
  Port[PortAddr+CtrlReg] := Init or Select;      {Restore 'default' state}
end.

