program NEWRATE;
(*
  Change samplerate of given file.

  Format of the wave file is as follows:
  'RIFF'                                (text)
  filesize-8                            (4 bytes)
  'WAVE'                                (text)
  'fmt '                                (text)
  infosize     (=$00000010)             (4 bytes)
  format       (PCM=$0001)              (2 bytes)
  channels     (stereo=$0002)           (2 bytes)
  samples/sec  (44100=$0000AC44)        (4 bytes)
  bytes/sec    (44100*4=$0002B110)      (4 bytes)
  bytes/sample (=$0004)                 (2 bytes)
  bits/sample  (=$0010)                 (2 bytes)
  'data'                                (text)
  datasize     (=filesize-44)           (4 bytes)
  ....sample data....


  Note: a 4 bytes number $12345678 is saved as  $78 $56 $34 $12
        a 2 bytes number $1234     is saved as  $34 $12

*)
{$M 16000, 32768, 32768}
{$X+}   { Function results not necessarily used }

uses Crt, Dos;

var
  Size        : longint;
  Rate        : longint;
  ByteFile    : file of byte;
  FileName    : string;
  Error       : integer;
  AByte       : byte;

begin
  if ParamCount<2 then
    begin
      writeln('Use   NEWRATE filename samplerate.');
      writeln('Standard samplerates are: 32000, 44100, 48000.');
      HALT;
    end;
  FileName   := ParamStr(1);

  val(ParamStr(2), Rate, Error);
  if Error<>0 then
  begin
    writeln('Samplerate is no number.');
    HALT;
  end;
  Assign(ByteFile, FileName);
  {$I-}
  reset(ByteFile);
  {$I+}
  if IoResult<>0 then
  begin
    writeln('File error.');
    HALT;
  end;


  Size := Rate;                                 { Overwrite samples/sec }
  seek(ByteFile, 24);
  AByte := Size and $FF;
  write(ByteFile, AByte);
  AByte := (Size and $FF00) shr 8;
  write(ByteFile, AByte);
  AByte := (Size and $FF0000) shr 16;
  write(ByteFile, AByte);
  AByte := (Size and $FF000000) shr 24;
  write(ByteFile, AByte);

  Size := Rate * 4;                             { Overwrite bytes/sec }
  seek(ByteFile, 28);
  AByte := Size and $FF;
  write(ByteFile, AByte);
  AByte := (Size and $FF00) shr 8;
  write(ByteFile, AByte);
  AByte := (Size and $FF0000) shr 16;
  write(ByteFile, AByte);
  AByte := (Size and $FF000000) shr 24;
  write(ByteFile, AByte);

  close(ByteFile);

end.
